// Generated by CoffeeScript 1.4.0
var Element, animate, classList, css, doc, elementFactory, elements, event, id, numberUtils, objectUtils, select, text, win;

classList = require('./classlist');

select = require('./select');

text = require('./text');

css = require('./css');

event = require('../event');

objectUtils = require('../utils/objectutils');

numberUtils = require('../utils/numberutils');

animate = require('../utils/animate');

win = window;

doc = win.document;

elements = [];

id = 0;

elementFactory = function(domElement) {
  var el, item, _i, _len;
  for (_i = 0, _len = elements.length; _i < _len; _i++) {
    item = elements[_i];
    if (item.domElement === domElement) {
      return item;
    }
  }
  el = new Element(domElement);
  elements.push(el);
  return el;
};

module.exports = function(selector, context, tag) {
  var element, _i, _len, _results;
  if (context == null) {
    context = doc;
  }
  if (objectUtils.isString(selector)) {
    selector = select(selector, context, tag);
  } else if (!(objectUtils.isElement(selector) || objectUtils.isArray(selector))) {
    throw 'not a valid element object or selector';
  }
  if (objectUtils.isArray(selector)) {
    _results = [];
    for (_i = 0, _len = selector.length; _i < _len; _i++) {
      element = selector[_i];
      if (objectUtils.isElement(element)) {
        _results.push(elementFactory(element));
      }
    }
    return _results;
  } else if (selector != null) {
    return elementFactory(selector);
  } else {
    return null;
  }
};

Element = (function() {

  function Element(domElement) {
    this.domElement = domElement;
    this.anim = null;
    this.id = this.domElement.id;
    this._id = id++;
    this.tagName = this.domElement.tagName;
  }

  Element.prototype.width = function(value) {
    if (value != null) {
      this.setStyle('width', value);
      if (this.domElement.width != null) {
        this.domElement.width = value;
      }
      return this;
    } else {
      return this.domElement.offsetWidth;
    }
  };

  Element.prototype.height = function(value) {
    if (value != null) {
      this.setStyle('height', value);
      if (this.domElement.height != null) {
        this.domElement.height = value;
      }
      return this;
    } else {
      return this.domElement.offsetHeight;
    }
  };

  Element.prototype.opacity = function(value) {
    if (value != null) {
      this.setStyle('opacity', numberUtils.limit(parseFloat(value), 0, 1));
      return this;
    } else {
      return this.getStyle('opacity');
    }
  };

  Element.prototype.offset = function() {
    return {
      top: this.domElement.offsetTop,
      left: this.domElement.offsetLeft
    };
  };

  Element.prototype.position = function() {
    var el, left, top;
    top = this.domElement.offsetTop;
    left = this.domElement.offsetLeft;
    if ((el = this.domElement).offsetParent) {
      while ((el = el.offsetParent)) {
        top += el.offsetTop;
        left += el.offsetLeft;
      }
    }
    return {
      top: top,
      left: left
    };
  };

  Element.prototype.hasClass = function(clas) {
    return classList.hasClass(this.domElement, clas);
  };

  Element.prototype.matchClass = function(clas) {
    return classList.matchClass(this.domElement, clas);
  };

  Element.prototype.addClass = function(clas) {
    classList.addClass(this.domElement, clas);
    return this;
  };

  Element.prototype.removeClass = function(clas) {
    classList.removeClass(this.domElement, clas);
    return this;
  };

  Element.prototype.toggleClass = function(clas) {
    classList.toggleClass(this.domElement, clas);
    return this;
  };

  Element.prototype.replaceClass = function(clasOld, clasNew) {
    classList.replaceClass(this.domElement, clasOld, clasNew);
    return this;
  };

  Element.prototype.addTemporaryClass = function(clas, duration) {
    classList.addTemporaryClass(this.domElement, clas, duration);
    return this;
  };

  Element.prototype.getText = function() {
    return text.getText(this.domElement);
  };

  Element.prototype.setText = function(value) {
    text.setText(this.domElement, value);
    return this;
  };

  Element.prototype.getHTML = function() {
    return this.domElement.innerHTML;
  };

  Element.prototype.setHTML = function(value) {
    this.domElement.innerHTML = value;
    return this;
  };

  Element.prototype.getStyle = function(property) {
    return css.getStyle(this.domElement, property);
  };

  Element.prototype.getNumericStyle = function(property) {
    return css.getNumericStyle(this.domElement, property);
  };

  Element.prototype.setStyle = function(property, value) {
    css.setStyle(this.domElement, property, value);
    return this;
  };

  Element.prototype.clearStyle = function(property) {
    css.clearStyle(this.domElement, property);
    return this;
  };

  Element.prototype.on = function(type, callback, data) {
    event.on(this, type, callback, data);
    return this;
  };

  Element.prototype.off = function(type, callback) {
    event.off(this, type, callback);
    return this;
  };

  Element.prototype.one = function(type, callback, data) {
    event.one(this, type, callback, data);
    return this;
  };

  Element.prototype.animate = function() {
    var _ref;
    return (_ref = this.anim) != null ? _ref : this.anim = animate(this.domElement, true);
  };

  Element.prototype.getAttribute = function(type) {
    return this.domElement.getAttribute(type);
  };

  Element.prototype.setAttribute = function(type, value) {
    this.domElement.setAttribute(type, value);
    return this;
  };

  Element.prototype.find = function(selector) {
    return module.exports(selector, this);
  };

  Element.prototype.parent = function(asElement) {
    if (asElement == null) {
      asElement = true;
    }
    if (asElement) {
      return new Element(this.domElement.parentNode);
    } else {
      return this.domElement.parentNode;
    }
  };

  Element.prototype.children = function(asElement) {
    var child, _i, _len, _ref, _results;
    if (asElement == null) {
      asElement = true;
    }
    _ref = this.domElement.childNodes;
    _results = [];
    for (_i = 0, _len = _ref.length; _i < _len; _i++) {
      child = _ref[_i];
      if ((child != null ? child.nodeType : void 0) === 1) {
        _results.push(asElement ? new Element(child) : child);
      }
    }
    return _results;
  };

  Element.prototype.firstChild = function(asElement) {
    if (asElement == null) {
      asElement = true;
    }
    if (asElement) {
      return new Element(this.domElement.firstChild);
    } else {
      return this.domElement.firstChild;
    }
  };

  Element.prototype.lastChild = function(asElement) {
    if (asElement == null) {
      asElement = true;
    }
    if (asElement) {
      return new Element(this.domElement.lastChild);
    } else {
      return this.domElement.lastChild;
    }
  };

  Element.prototype.appendChild = function(element) {
    return this.domElement.appendChild(element.domElement || element);
  };

  Element.prototype.removeChild = function(element) {
    return this.domElement.removeChild(element.domElement || element);
  };

  Element.prototype.replaceChild = function(newElement, oldElement) {
    this.domElement.replaceChild(newElement.domElement || newElement, oldElement.domElement || oldElement);
    return oldElement;
  };

  Element.prototype.remove = function() {
    return this.parent().removeChild(this.domElement);
  };

  Element.prototype.insertBefore = function(element, referenceElement) {
    return this.domElement.insertBefore(element.domElement || element, referenceElement.domElement || referenceElement);
  };

  Element.prototype.clone = function(deep, asElement) {
    if (asElement == null) {
      asElement = true;
    }
    if (asElement) {
      return new Element(this.domElement.clone(deep));
    } else {
      return this.domElement.clone(deep);
    }
  };

  Element.prototype.destroy = function(remove) {
    var idx, item, _i, _len, _results;
    if (remove == null) {
      remove = false;
    }
    event.offAll(this);
    if (this.anim != null) {
      if (this.anim.isRunning) {
        this.anim.keep = false;
      } else {
        this.anim.destroy();
      }
      this.anim = null;
    }
    if (remove) {
      this.domElement.parentNode.removeChild(this.domElement);
    }
    this.domElement = null;
    _results = [];
    for (idx = _i = 0, _len = elements.length; _i < _len; idx = ++_i) {
      item = elements[idx];
      if (item === this) {
        elements.splice(idx, 1);
      }
      break;
    }
    return _results;
  };

  return Element;

})();
