// Generated by CoffeeScript 1.4.0
var DomEvent, Event, capabilities, createHandler, doc, domListeners, element, getType, id, isValidElement, platform, removeCachedListener, win, _ref;

_ref = require('./environment'), platform = _ref.platform, capabilities = _ref.capabilities;

element = null;

win = window;

doc = win.document;

domListeners = [];

id = 0;

isValidElement = function(element) {
  return !!(element && ((element.domElement != null) || element === win || element.nodeType === 9 || element.nodeType === 1));
};

createHandler = function(target, type, data, callback) {
  return function(event) {
    return callback(new DomEvent(event, target, type, data));
  };
};

removeCachedListener = function(target, type, callback) {
  var i, item, _i, _len;
  for (i = _i = 0, _len = domListeners.length; _i < _len; i = ++_i) {
    item = domListeners[i];
    if ((item.target.domElement === target.domElement || item.target === target) && item.type === type && item.callback === callback) {
      return domListeners.splice(i, 1)[0];
    }
  }
};

getType = function(type) {
  if (capabilities.hasTouch()) {
    switch (type) {
      case 'mousedown':
        type = 'touchstart';
        break;
      case 'mousemove':
        type = 'touchmove';
        break;
      case 'mouseup':
        type = 'touchend';
    }
  }
  return type;
};

exports.on = function(target, type, callback, data) {
  var handler, _ref1;
  if (element == null) {
    element = require('./dom/element');
  }
  if (isValidElement(target)) {
    handler = createHandler(target, type, data, callback);
    handler.id = id++;
    domListeners.push({
      target: target,
      type: type,
      handler: handler,
      callback: callback
    });
    if (target.domElement) {
      target = target.domElement;
    }
    type = getType(type);
    if (target.addEventListener) {
      target.addEventListener(type, handler, false);
    } else if (target.attachEvent) {
      target.attachEvent("on" + type, handler);
    }
  } else {
    if ((_ref1 = target._handlers) == null) {
      target._handlers = {};
    }
    if (!(type in target._handlers)) {
      target._handlers[type] = [];
    }
    target._handlers[type].push(callback);
  }
  return target;
};

exports.one = function(target, type, callback, data) {
  var cb,
    _this = this;
  exports.on(target, type, cb = function(event) {
    callback.call(null, event);
    return exports.off(target, type, cb);
  }, data);
  return target;
};

exports.off = function(target, type, callback) {
  var cb, i, listener, _i, _len, _ref1;
  if (isValidElement(target)) {
    if (listener = removeCachedListener(target, type, callback)) {
      if (target.domElement) {
        target = target.domElement;
      }
      type = getType(type);
      if (target.removeEventListener) {
        target.removeEventListener(type, listener.handler, false);
      } else if (target.detachEvent) {
        target.detachEvent("on" + type, listener.handler);
      }
    }
  } else {
    _ref1 = target._handlers[type];
    for (i = _i = 0, _len = _ref1.length; _i < _len; i = ++_i) {
      cb = _ref1[i];
      if (callback === cb) {
        target._handlers[type].splice(i, 1);
        break;
      }
    }
  }
  return target;
};

exports.offAll = function(target) {
  var listener, _i, _len;
  if (isValidElement(target)) {
    for (_i = 0, _len = domListeners.length; _i < _len; _i++) {
      listener = domListeners[_i];
      if (target === (listener.target.domElement || listener.target)) {
        exports.off(listener.target, listener.type, listener.callback);
      }
    }
  } else {
    target._handlers = null;
  }
  return target;
};

exports.trigger = function(target, type, data) {
  var callback, evt, list, _i, _len;
  if (!isValidElement(target)) {
    if (target._handlers && type in target._handlers) {
      list = target._handlers[type].slice();
      evt = new Event(target, type, data);
      if (list.length === 1) {
        list[0].call(target, evt);
      } else {
        for (_i = 0, _len = list.length; _i < _len; _i++) {
          callback = list[_i];
          callback.call(target, evt);
        }
      }
      return target;
    }
  }
};

exports.dispatcher = function(target) {
  if (!isValidElement(target)) {
    target['on'] = function(type, callback) {
      return exports.on(this, type, callback);
    };
    target['off'] = function(type, callback) {
      return exports.off(this, type, callback);
    };
    target['one'] = function(type, callback) {
      return exports.one(this, type, callback);
    };
    return target['trigger'] = function(type, data) {
      return exports.trigger(this, type, data);
    };
  }
};

Event = (function() {

  function Event(target, type, data) {
    this.target = target;
    this.type = type;
    this.data = data;
  }

  return Event;

})();

DomEvent = (function() {

  function DomEvent(event, target, type, data) {
    var pos;
    this.type = type;
    this.data = data;
    this.domEvent = event || win.event;
    this.currentTarget = target;
    this.target = this.domEvent.target || this.domEvent.srcElement || win;
    if (this.target.nodeType === 3) {
      this.target = this.target.parentNode;
    }
    if (!this.target.domElement && this.target.nodeType === 1) {
      this.target = element(this.target);
    }
    this.rightClick = this.domEvent.which ? this.domEvent.which === 3 : this.domEvent.button ? this.domEvent.button === 2 : false;
    this.leftClick = this.domEvent.which ? this.domEvent.which === 1 : this.domEvent.button ? this.domEvent.button === 0 || this.domEvent.button === 1 : true;
    if (this.type === 'mousedown' || this.type === 'mousemove') {
      if (this.domEvent.touches) {
        this.touches = this.domEvent.touches;
        if (this.touches.length) {
          this.pageX = this.touches[0].pageX;
          this.pageY = this.touches[0].pageY;
        }
      } else {
        this.pageX = this.domEvent.pageX != null ? this.domEvent.pageX : this.domEvent.clientX + doc.body.scrollLeft + doc.documentElement.scrollLeft;
        this.pageY = this.domEvent.pageY != null ? this.domEvent.pageY : this.domEvent.clientY + doc.body.scrollTop + doc.documentElement.scrollTop;
      }
      if ((this.domEvent.offsetX != null) && (this.domEvent.offsetY != null) && (this.domEvent.offsetX !== 0 && this.domEvent.offsetY !== 0)) {
        this.x = this.domEvent.offsetX;
        this.y = this.domEvent.offsetY;
      } else if ((this.domEvent.x != null) && (this.domEvent.y != null)) {
        this.x = this.domEvent.x;
        this.y = this.domEvent.y;
      } else {
        pos = this.target.domElement ? this.target.position() : {
          left: this.target.offsetLeft,
          top: this.target.offsetTop
        };
        this.x = this.pageX ? this.pageX - pos.left : 0;
        this.y = this.pageY ? this.pageY - pos.top : 0;
      }
    }
  }

  DomEvent.prototype.preventDefault = function() {
    if (this.domEvent.preventDefault) {
      return this.domEvent.preventDefault();
    } else {
      return this.domEvent.returnValue = false;
    }
  };

  DomEvent.prototype.stopPropagation = function() {
    if (this.domEvent.stopPropagation) {
      return this.domEvent.stopPropagation();
    } else {
      return this.domEvent.cancelBubble = true;
    }
  };

  DomEvent.prototype.stop = function() {
    this.preventDefault();
    return this.stopPropagation();
  };

  return DomEvent;

})();

if ((win != null) && win.attachEvent && !win.addEventListener) {
  win.attachEvent('onunload', function() {
    var listener, _i, _len, _results;
    _results = [];
    for (_i = 0, _len = domListeners.length; _i < _len; _i++) {
      listener = domListeners[_i];
      try {
        _results.push(exports.off(listener.target, listener.type, listener.callback));
      } catch (e) {

      }
    }
    return _results;
  });
}
