// Generated by CoffeeScript 1.4.0
var Anim, DEFAULT_DURATION, FRAME_RATE, add, animate, anims, css, csstransitions, destroy, doc, easing, last, length, objectUtils, polyfill, pool, remove, running, tick, uid, win, _onTick,
  __bind = function(fn, me){ return function(){ return fn.apply(me, arguments); }; },
  __slice = [].slice;

polyfill = require('./polyfill');

easing = require('./easing');

css = require('../dom/css');

objectUtils = require('./objectutils');

win = window;

doc = win.document;

FRAME_RATE = 60;

DEFAULT_DURATION = 0.5;

anims = {};

length = 0;

pool = [];

uid = 1;

tick = 0;

last = 0;

running = false;

csstransitions = css.csstransitions;

animate = module.exports = function(target, keep) {
  var anim;
  if (keep == null) {
    keep = false;
  }
  anim = pool.length ? pool.pop() : new Anim;
  anim.id = uid++;
  anim.target = target;
  anim.keep = keep;
  return anim;
};

add = function(anim) {
  if (!anims[anim.id]) {
    anims[anim.id] = anim;
    anim.isRunning = true;
    length++;
    if (!running) {
      running = true;
      tick = 0;
      last = +(new Date);
      return _onTick();
    }
  }
};

remove = function(anim) {
  anim.isRunning = false;
  delete anims[anim.id];
  length--;
  if (!length) {
    return running = false;
  }
};

destroy = function(anim) {
  if (anim.id) {
    if (anim.isRunning) {
      remove(anim);
    }
    anim.id = null;
    anim.target = null;
    anim.duration = 0;
    anim.elapsed = 0;
    anim.properties = {};
    anim.easing = easing['outCubic'];
    anim.tickCallbacks = [];
    anim.completeCallbacks = [];
    anim.keep = false;
    anim.isRunning = false;
    anim.useCssTransitions = false;
    anim.transitionStyle = '';
    return pool.push(anim);
  }
};

_onTick = function(time) {
  var anim, id, now;
  now = +(new Date);
  tick = now - last;
  last = now;
  for (id in anims) {
    anim = anims[id];
    anim._render(tick);
  }
  if (running) {
    return win.requestAnimationFrame(_onTick);
  }
};

Anim = (function() {

  function Anim() {
    this._render = __bind(this._render, this);
    this.id = null;
    this.target = null;
    this.duration = 0;
    this.elapsed = 0;
    this.properties = {};
    this.easing = easing['outCubic'];
    this.tickCallbacks = [];
    this.completeCallbacks = [];
    this.isRunning = false;
    this.keep = false;
    this.useCssTransitions = false;
    this.transitionStyle = '';
  }

  Anim.prototype.to = function(properties, duration, ease) {
    var current, end, p, prop, s1, val;
    if (duration == null) {
      duration = DEFAULT_DURATION;
    }
    this.duration = duration * 1000;
    this.elapsed = 0;
    if (ease != null) {
      this.easing = easing[ease];
    }
    this.properties = {};
    this.useCssTransitions = false;
    for (prop in properties) {
      val = properties[prop];
      p = {
        start: 0,
        current: 0,
        end: val,
        type: 0
      };
      if (objectUtils.isFunction(this.target[prop])) {
        p.start = this.target[prop]();
        p.type = 1;
      } else if (prop in this.target) {
        p.start = this.target[prop];
        p.type = 2;
      } else {
        current = css.getNumericStyle(this.target, prop);
        p.start = current[0];
        if (objectUtils.isString(val)) {
          end = css.parseNumber(val, prop);
          p.unit = end[1];
          p.end = end[0];
        } else {
          p.unit = current[1];
          p.end = val;
        }
        if (csstransitions) {
          if (!this.useCssTransitions) {
            s1 = (this.target.getAttribute('style') || '').length;
            css.setStyle(this.target, {
              'transition-property': 'all',
              'transition-duration': "" + this.duration + "ms"
            });
            if (ease) {
              css.setStyle(this.target, 'transition-timing-function', easing.css[ease]);
            }
            this.transitionStyle = this.target.getAttribute('style').slice(s1);
            this.useCssTransitions = true;
          }
          p.type = 4;
          css.setStyle(this.target, prop, p.end + p.unit);
        } else {
          p.type = 3;
        }
      }
      this.properties[prop] = p;
    }
    add(this);
    return this;
  };

  Anim.prototype.getProperty = function(property) {
    var _ref;
    if (this.isRunning) {
      return (_ref = this.properties[property]) != null ? _ref.current : void 0;
    }
  };

  Anim.prototype.setProperty = function(property, value) {
    var _ref;
    if (this.isRunning) {
      return (_ref = this.properties[property]) != null ? _ref.end = value : void 0;
    }
  };

  Anim.prototype.onTick = function() {
    var args, callback;
    callback = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
    this.tickCallbacks.push({
      func: callback,
      args: args
    });
    return this;
  };

  Anim.prototype.onComplete = function() {
    var args, callback;
    callback = arguments[0], args = 2 <= arguments.length ? __slice.call(arguments, 1) : [];
    this.completeCallbacks.push({
      func: callback,
      args: args
    });
    return this;
  };

  Anim.prototype.stop = function() {
    if (this.isRunning) {
      if (this.keep) {
        return remove(this);
      } else {
        return destroy(this);
      }
    }
  };

  Anim.prototype.destroy = function() {
    destroy(this);
    return null;
  };

  Anim.prototype._render = function(time) {
    var b, c, callback, cbs, dur, prop, propObj, value, _i, _len, _ref, _ref1;
    this.elapsed += time;
    dur = this.elapsed < this.duration ? this.elapsed : this.duration;
    _ref = this.properties;
    for (prop in _ref) {
      propObj = _ref[prop];
      if (propObj.type < 4) {
        b = propObj.start;
        c = propObj.end - b;
        value = propObj.current = this.easing(dur, b, c, this.duration);
        switch (propObj.type) {
          case 1:
            this.target[prop](value);
            break;
          case 2:
            this.target[prop] = value;
            break;
          case 3:
            css.setStyle(this.target, prop, "" + value + propObj.unit);
        }
      }
    }
    if (this.tickCallbacks.length) {
      if (this.tickCallbacks.length === 1) {
        this.tickCallbacks[0].func.apply(null, this.tickCallbacks[0].args);
      } else {
        _ref1 = this.tickCallbacks;
        for (_i = 0, _len = _ref1.length; _i < _len; _i++) {
          callback = _ref1[_i];
          callback.func.apply(null, callback.args);
        }
      }
    }
    if (this.elapsed >= this.duration) {
      if (this.useCssTransitions) {
        this.target.setAttribute('style', this.target.getAttribute('style').replace(this.transitionStyle, ''));
        this.useCssTransitions = false;
        this.transitionStyle = '';
      }
      cbs = this.completeCallbacks.slice();
      this.tickCallbacks = [];
      this.completeCallbacks = [];
      this.properties = {};
      if (this.keep) {
        remove(this);
      } else {
        destroy(this);
      }
      if (cbs.length) {
        return win.requestAnimationFrame(function() {
          var _j, _len1;
          if (cbs.length === 1) {
            cbs[0].func.apply(null, cbs[0].args);
          } else {
            for (_j = 0, _len1 = cbs.length; _j < _len1; _j++) {
              callback = cbs[_j];
              callback.func.apply(null, callback.args);
            }
          }
          return cbs = null;
        });
      }
    }
  };

  return Anim;

})();
